/**
 * @file       servo.h
 * @brief      Manages 24 servo motors
 * 
 * @author     laurent Saint-Marcel(lstmarcel@yahoo.fr)
 * @date       2002/09/02
 */

#ifndef __SERVO_MOTORS_24_H__
#define __SERVO_MOTORS_24_H__

#include "baseTypes.h"

/**
 * @brief	Minimal servo position
 * @see		servoSetPos
 */
#define SERVO_MIN_POS		185
/**
 * @brief	Maximal servo position
 * @see		servoSetPos
 */
#define SERVO_MAX_POS		245

#define SERVO_NUMBER		24
#define SERVO_MASK_NUMBER	4
/**
 * @brief   This function sets a servo to the corresponding position
 *
 * @param   servoId [0..23]
 * @param   pos     [SERVO_MIN_POS..SERVO_MAX_POS]
 * @see     SERVO_MIN_POS
 * @see     SERVO_MAX_POS
 */
void servoSetPos(BYTE servoId, 
                 BYTE pos);

/**
 * @brief   This function sets the servo mask that enable/disable 
 *          servo-motors. It corresponds to 8 servos or 4 servos in the same time
 *
 * @param   boardId [0..1] 0=>servoId=[0..11], 1=>servoId=[12..23]
 * @param   port    [0..1] 0=>servoId=[0..7]+[12..19], 1=>servoId=[8..11]+[20..23]    
 * @param   mask    0=disable, 1=enable corresponding servo-motor
 * @see     servoEnable
 * @see     servoIsEnable
 */
void servoSetMask(BYTE boardId, 
                  BYTE port, 
                  BYTE mask);

/**
 * @brief   This function enable/disable one servo-motor
 *
 * @param   servoId [0..23]
 * @param   enable  TRUE => enable, FALSE => disable
 * @see     servoSetMask
 * @see     servoIsEnable
 */
void servoEnable(BYTE servoId, 
                 BOOL enable);

/**
 * @brief   This function returns true if a servo is activated (enable)
 *
 * @param   servoId [0..23]
 * @return  TRUE => enable, FALSE => disable
 * @see     servoSetMask
 * @see		servoEnable
 */
BOOL servoIsEnable(BYTE servoId);

/**
 * @brief   This function wait for servo to reach their position
 *
 * @param   servoStep : number of step between current servo
 *          position and target position
 */
void servoWait(BYTE servoStep);

#endif /* __SERVO_MOTORS_24_H__ */
