;=========================================
; Program to control 12 servo motors 
; through serial port
;-----------------------------------------
; 2002/07/28  --  lstmarcel@yahoo.fr
; Http://laurent.saintmarcel.free.fr
; Target = at90s2313, 8MHz
; Compiled with avr studio from Atmel
;=========================================

.include "2313def.inc"

;------------------------------------------------------------------
;   output = PORTB (servo addr:[0..7])
;            PORTD (pins[2..5], servo addr:[8..11])
;            PORTD  pin 0=RX connected, 
;                   pin 1=TX not connected, 
;                   pin 6=serial port busy: don't send serial data when this pin is hight
;------------------------------------------------------------------
;
; communication protocol :
; board id =0x00 or 0x20
; Set servo Pos= 0=> set enableMask, mask = data, 
;                    if addr3==0, data = mask for lower servo ids else higher servo ids
; Set servo Pos= 1=> set servo pos using data
;  --------------------------------------------------------------------------------------
;  | Board ID2 | Board ID1 | Board ID0 | Set servo Pos | addr3 | addr2 | addr1 | addr 0 |
;  --------------------------------------------------------------------------------------
;  -----------------------------------------------------------------
;  | data7 | data6 | data5 | data4 | data3 | data2 | data1 | data0 |
;  -----------------------------------------------------------------
;
;------------------------------------------------------------------
;
;*****************  USER PARAMETERS *********************
;
;======================
; configure board ID
;======================
;.equ	boardID		=0x00  ;0
.equ	boardID		=0x20  ;1

;=============================
; set default servo position
;=============================
.equ	defaultServoPos =215	; 190..240

;------------------------------------------------------------------

;*********************  PROGRAM *************************

;=====================
; communication info
;=====================
.equ	boardBit	=0xe0 
.equ	servoSetBit	=0x10
.equ	servo0Adress	=0x08

.equ	servoData1	=RAMSTART
.equ	servoData0	=RAMSTART+8

.equ	LONG_TIMER1	=50
.equ	LONG_TIMER0	=10

.equ	pinSciAck	=6     ; pin on PORTD that is set to one 
;                                when receiving data and cleared 
;                                when SCI is ready to receive data
;========================
; variables
;========================
;.equ    RAMSTART=$60    ;First On-Chip SRAM Location
.def	outputreg   = r25
.def    serialData  = r24
.def 	serialData2 = r26
.def 	serialCount = r27

;r28 & r29 reserved to YL and YH

.def	header	    = r23	; communication header 
.def	Tmp	    = r22
.def	Tmp0	    = r21
.def	Tmp1	    = r20
.def	servoMask1  = r19
.def	servoMask0  = r18
.def	servoBit1   = r17
.def	servoBit0   = r16
.def	addrCount0  = r0
.def	addrCount1  = r1
.def	portD0	    = r3
;===========================
; 	EEPROM
;===========================
.eseg					;EEPROM segment
.org 0

;===========================
;	SOURCE CODE
;===========================	
.cseg	;CODE segment
.org 	$000 ; reset
	rjmp __reset_
.org	INT0addr 	;/* External Interrupt0 Vector Address */
	reti
.org	INT1addr 	;/* External Interrupt1 Vector Address */
	reti
.org	ICP1addr 	;/* Input Capture1 Interrupt Vector Address */
	reti
.org	OC1addr 	;/* Output Compare1 Interrupt Vector Address */
	reti
.org	OVF1addr 	;/* Overflow1 Interrupt Vector Address */
	rjmp __timer1_overflow_
.org	OVF0addr 	;/* Overflow0 Interrupt Vector Address */
	rjmp __timer0_overflow_
.org	URXCaddr 	;/* UART Receive Complete Interrupt Vector Address */
	reti
.org	UDREaddr 	;/* UART Data Register Empty Interrupt Vector Address */
	reti
.org	UTXCaddr 	;/* UART Transmit Complete Interrupt Vector Address */
	reti
.org	ACIaddr  	;/* Analog Comparator Interrupt Vector Address */
	reti
	
	
;========================
; program setup
;========================	
__reset_:
	ldi	Tmp,RAMEND	; Set initial stack ptr location at ram end
	out	SPL,Tmp
	clr	YH

; define ports direction
	ldi 	Tmp, 0xff	; PORT B = all ouput:servos
	out 	DDRB,Tmp
	ldi 	Tmp, 0xfc	; PORT C = 0,1=serial port, others(2..5) = servos, 6=SCI received ready,
	out 	DDRD, Tmp
; servo informations
	ldi 	servoBit0,  0x01
	ldi 	servoBit1,  0x01
	ldi 	servoMask0, 0x00 ; servo disabled ; 0x0f =enable
	ldi 	servoMask1, 0x00 ; servo disabled ; 0xff =enable
	clr 	addrCount0
	clr 	addrCount1
	rcall 	servoPos_init
; initialise serial port:
	rcall 	sci_init

; configure timer interrupts
	ldi 	Tmp,0x04	; CK/256
	out 	TCCR0, Tmp
	out 	TCCR1B, Tmp
	; initialise le compteur  la bonne valeur
	ldi	Tmp, LONG_TIMER0
	out 	TCNT0, Tmp
	ldi	Tmp, LONG_TIMER1
	out 	TCNT1L, Tmp
	ldi	Tmp, 0xff
	out 	TCNT1H, Tmp
	;enable timer interrupts
	ldi 	Tmp,0x82
	out 	TIMSK, Tmp
;enable interupts
	sei
; jump to real begining
	rjmp 	main
	
;========================
;  SERIAL PORT  
;========================
;**** Init serial port  ****
sci_init:	
	ldi  Tmp, 0x18	        ; RX/TX enable, 8bits, RX interruptions
	out  UCR, Tmp
	ldi  Tmp, 51	        ; 9600 baud for a 8MHz quartz
	out  UBRR, Tmp 	        ; set baud rate 
	ret
;**** send one byte on serial port : register 'serialData' ****
sci_snd:
	sbis USR, UDRE 		; Write Data register empty ?
	rjmp sci_snd 		; if No => retry
	out  UDR, serialData	; write data on serial port
sci_snde:
	sbis USR, TXC 		; Transfer Data complete ?
	rjmp sci_snde 		; if No => retry	
	ret
;**** wait to receive one byte from serial port and store it in register 'serialData' ****
sci_rcv:
	sbis USR, RXC 		; Data received
	rjmp sci_rcv		; if No => retry
	in   serialData, UDR	; get data
;	clr  serialCount
sci_rcv2:
;	inc  serialCount	; if wait too long for other data that means there is a problem => restart waiting
;	breq sci_rcv
	sbis USR, RXC 		; Data received
	rjmp sci_rcv2		; if No => retry
	in   serialData2, UDR	; get data
	ret
;
;========================
; servo pos
;========================
servoPos_init:
	ldi	YL,servoData1		; move cursor to servo time
	ldi	Tmp, defaultServoPos	; default servo pos
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	ldi	YL,servoData0		; move cursor to servo time
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	st	Y+, Tmp
	ret
;
;=====================================
; servo control using tiner interrupt
;=====================================	
;**** first 8 servo motors use timer1 and port B (b0..b7)***
__timer1_overflow_:
	ldi	Tmp1, 0x00
	out	PORTB, Tmp1		; OFF all servo on portB => stop previous order
endTimer1Overflow1:
	cpi	servoBit1, 0x00		;if servobit == last bit then wait a special time
	brne	endTimer1Overflow2	
	ldi	Tmp1, LONG_TIMER1
	out	TCNT1L, Tmp1
	ldi	Tmp1, 0xff
	out	TCNT1H, Tmp1
	ldi	Tmp1, 0x00		; reset datas
	mov	addrCount1, Tmp1
	ldi	servoBit1, 0x01		
	rjmp	endTimer1OverflowEND
endTimer1Overflow2:
	mov 	Tmp1, servoBit1		; get next servo
	and	Tmp1, servoMask1	; set unable or disable
	cp	Tmp1, servoBit1
	brne	endTimer1Overflow3	; if current servo must be OFF just prepare next data
	out	PORTB, Tmp1		; ON only one servo corresponding to servoBit1
endTimer1Overflow3:
	push	YL
	ldi	YL,servoData1		; move cursor to servo time
	add	YL, addrCount1
	ld	Tmp1, Y			; get time corresponding to cursor
	out	TCNT1L, Tmp1		; set next timer time to this value
	ldi	Tmp1, 0xff
	out	TCNT1H, Tmp1
	pop	YL
; prepare next data
	inc	addrCount1
	rol	servoBit1		; shift bit on left
endTimer1OverflowEND:
	reti	
	
;**** 4 others servo motors use timer0, and portD (b2..b5)***	
__timer0_overflow_:
	cbi	PORTD, 2
	cbi	PORTD, 3
	cbi	PORTD, 4
	cbi	PORTD, 5
endTimer0Overflow1:
	cpi	servoBit0, 0x20		; if servobit == last bit then wait a special time
	brne	endTimer0Overflow2
	ldi	Tmp0, LONG_TIMER0
	out	TCNT0, Tmp0
	ldi	Tmp0, 0x00		; reset data
	mov	addrCount0, Tmp0
	ldi	servoBit0, 0x01
	rjmp	endTimer0OverflowEND
endTimer0Overflow2:
	mov 	Tmp0, servoBit0
	and	Tmp0, servoMask0
	cp	Tmp0, servoBit0
	brne	endTimer0Overflow3
	cpi	Tmp0, 0x01
	brne	endTimer0Overflow22	
	sbi	PORTD, 2
	rjmp	endTimer0Overflow3
endTimer0Overflow22:
	cpi	Tmp0, 0x02
	brne	endTimer0Overflow23	
	sbi	PORTD, 3
	rjmp	endTimer0Overflow3
endTimer0Overflow23:
	cpi	Tmp0, 0x04
	brne	endTimer0Overflow24	
	sbi	PORTD, 4
	rjmp	endTimer0Overflow3
endTimer0Overflow24:
	cpi	Tmp0, 0x08
	brne	endTimer0Overflow3	
	sbi	PORTD, 5
endTimer0Overflow3:
	push	YL
	ldi	YL,servoData0	;
	add	YL, addrCount0
	ld	Tmp0, Y
	out	TCNT0, Tmp0
	pop	YL
	; prepare cursor to next data
	inc	addrCount0
	rol	servoBit0
endTimer0OverflowEND:
	reti
;
;===========================
; main program
;===========================
main:	
; get data from serial port
	rcall	sci_rcv
	mov	header, serialData
;	rcall	sci_snd
; compare board address	
	mov	Tmp, header
	andi	Tmp, boardBit
;	mov	serialData, Tmp
;	rcall	sci_snd
;	ldi	serialData, boardID
;	rcall	sci_snd
	cpi	Tmp, boardID
	breq	getUserCommand		; not this board id
;	ldi	serialData, 0x01
;	rcall	sci_snd
	rjmp	main
; user command	
getUserCommand:
	mov	Tmp, header
	andi	Tmp, servoSetBit; if bit servoSet NOT sets
	cpi	Tmp, servoSetBit 
	brne	setMask		; then it is setServoMask
				; else it is setServoPos
;****** SET servo pos *******
setServoPos:
	andi	header, servo0Adress
	cpi	header, servo0Adress
	brne	setServoPos1
setServoPos0:
	ldi	YL, servoData0	; get pointer to the begining of servo data
	andi	serialData,0x07 ; get part of the header corresponding to the address
	add	YL, serialData  ; move pointer to the right address
	st	Y, serialData2  ; store data
;	ldi	serialData, 0x02
;	rcall	sci_snd
	rjmp	main
setServoPos1:
	ldi	YL,servoData1	; get pointer to the begining of servo data
	andi	serialData,0x07 ; get part of the header corresponding to the address
	add	YL, serialData  ; move pointer to the right address
	st	Y, serialData2  ; store data
;	ldi	serialData, 0x03
;	rcall	sci_snd
	rjmp	main
;******* SET servo mask *******
setMask:
	andi	header, servo0Adress
	cpi	header, servo0Adress
	brne	setMask1
setMask0:
	mov	servoMask0, serialData2
;	ldi	serialData, 0x04
;	rcall	sci_snd
	rjmp	main
setMask1:
	mov	servoMask1, serialData2
;	ldi	serialData, 0x05
;	rcall	sci_snd
	rjmp	main	
; loop
	rjmp main