/*
 * /file   spiderleg.h
 * @brief  Functions that moves only one leg
 *
 * @description
 *
 * @author Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 *
 * @date   2002/09/15
 */
#include <stdafx.h>

#include "spiderleg.h"
#include "servo.h"
#include "spiderIO.h"

#ifdef LEG_USE_OPTIMIZED_DELAY

#ifdef WIN32
#include <math.h>
#endif /* WIN32 */

#endif /* LEG_USE_OPTIMIZED_DELAY */
/* ==================================
 *         SERVO POSITION & ID
 * ================================== */
/*
 * UP/DOWN
 */
BYTE legUp[LEG_NUMBER]          ={192,210,205, 208,210,208};
BYTE legMiddle[LEG_NUMBER]      ={202,220,218, 220,222,220};
BYTE legDown[LEG_NUMBER]        ={220,240,238, 240,242,240};
BYTE legUpDownServo[LEG_NUMBER] ={0,  4,  8,   12, 16, 20 };
#define LEG_DOWN_DELTA  -2  /* how much to move each step for "touch the ground",
			       leg is going down */
#define LEG_UP_DELTA     2 /* how much to move each step for "touch the ground",
			      leg is going down */
#define LEG_DOWN_MIN_FOR_GROUND  SERVO_MIN_POS+20
#define LEG_DOWN_MAX_FOR_GROUND  SERVO_MEDIUM_POS
/*
 * IN/OUT
 */
BYTE legIn[LEG_NUMBER]         ={245,240,245, 235,235,237};
BYTE legNormal[LEG_NUMBER]     ={220,215,220, 210,210,212};
BYTE legOut[LEG_NUMBER]        ={202,195,200, 190,190,192};
BYTE legInOutServo[LEG_NUMBER] ={1,  5,  9,   13, 17, 21 };

/*
 * RIGHT/LEFT
 */
BYTE legLeft[LEG_NUMBER]           ={200,210,195, 205,195,195};
BYTE legCenter[LEG_NUMBER]         ={215,225,210, 220,210,210};
BYTE legRight[LEG_NUMBER]          ={230,240,225, 235,225,225};
BYTE legRightLeftServo[LEG_NUMBER] ={2,  6,  10,  14, 18, 22 };


/* ==================================
 *               GLOBALS
 * ================================== */

/* servo current position, must be defined in servo.c */
extern BYTE servoPos[SERVO_NUMBER];

/* ==================================
 *              FUNCTIONS
 * ================================== */

/*
 * @brief This function returns a legId corresponding to
 * legId+LEG_NUMBER/2
 */
BYTE legGetOpposite(BYTE legId)
{
  return legId+LEG_NUMBER_2;
}


/*
 * @brief This function returns a valid legId between 0 and LEG_NUMBER
 */
BYTE legNormalize(BYTE legId)
{
  while(legId>LEG_NUMBER)
    legId-=LEG_NUMBER;
  return legId;
}


/*
 * @brief This function set the servos position for a leg
 * @param legId    leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                 will be corrected by legNormalize. Leg ids start to 0
 * @param servoUp  servo position. (LEG_DISABLE = disable servo, LEG_KEEP = don't send any
 *                 order to the servo, other = enable servo and set
 *                 corresponding position)
 */
BYTE legSetPosition(BYTE legId,
		    BYTE servoUp_,
		    BYTE servoRight_,
		    BYTE servoIn_)
{
#ifdef LEG_USE_OPTIMIZED_DELAY
  SBYTE deltaVar=0;
  BYTE deltaMax=0;
#endif /* LEG_USE_OPTIMIZED_DELAY */
  legId=legNormalize(legId);
  
  switch(servoUp_){
  case LEG_KEEP:
    /* do nothing */
    break;
  case LEG_DISABLE:
    /* disable servo if necessary */
    if (servoIsEnable(legUpDownServo[legId]))
      servoEnable(legUpDownServo[legId], FALSE);
    break;
  case LEG_ENABLE:
    /* disable servo if necessary */
    if (!servoIsEnable(legUpDownServo[legId]))
      servoEnable(legUpDownServo[legId], TRUE);
    break;
  default:
    /* set servo position and enable if necessary */
#ifdef LEG_USE_OPTIMIZED_DELAY
    deltaMax=abs(servoUp_-servoPos[legUpDownServo[legId]]);
#endif /* LEG_USE_OPTIMIZED_DELAY */
    servoSetPos(legUpDownServo[legId],
		servoUp_);
    if (!servoIsEnable(legUpDownServo[legId]))
      servoEnable(legUpDownServo[legId], TRUE);
	break;
  }
  
  switch(servoRight_){
  case LEG_KEEP:
    /* do nothing */
    break;
  case LEG_DISABLE:
    /* disable servo if necessary */
    if (servoIsEnable(legRightLeftServo[legId]))
      servoEnable(legRightLeftServo[legId], FALSE);
    break;
  case LEG_ENABLE:
    /* disable servo if necessary */
    if (!servoIsEnable(legRightLeftServo[legId]))
      servoEnable(legRightLeftServo[legId], TRUE);
    break;
  default:
    /* set servo position and enable if necessary */
#ifdef LEG_USE_OPTIMIZED_DELAY
    deltaVar=abs(servoRight_-servoPos[legUpDownServo[legId]]);
    if (deltaVar>deltaMax) deltaMax=deltaVar;
#endif /* LEG_USE_OPTIMIZED_DELAY */
    servoSetPos(legRightLeftServo[legId],
		servoRight_);
    if (!servoIsEnable(legRightLeftServo[legId]))
      servoEnable(legRightLeftServo[legId], TRUE);
	break;
  }
  
  switch(servoIn_){
  case LEG_KEEP:
    /* do nothing */
    break;
  case LEG_DISABLE:
    /* disable servo if necessary */
    if (servoIsEnable(legInOutServo[legId]))
      servoEnable(legInOutServo[legId], FALSE);
    break;
  case LEG_ENABLE:
    /* disable servo if necessary */
    if (!servoIsEnable(legInOutServo[legId]))
      servoEnable(legInOutServo[legId], TRUE);
    break;
  default:
    /* set servo position and enable if necessary */
#ifdef LEG_USE_OPTIMIZED_DELAY
    deltaVar=abs(servoIn_-servoPos[legInOutServo[legId]]);
    if (deltaVar>deltaMax) deltaMax=deltaVar;
#endif /* LEG_USE_OPTIMIZED_DELAY */
    servoSetPos(legInOutServo[legId],
		servoIn_);
    if (!servoIsEnable(legInOutServo[legId]))
      servoEnable(legInOutServo[legId], TRUE);
	break;
  }
#ifdef LEG_USE_OPTIMIZED_DELAY 
  return deltaMax;
#else /* LEG_USE_OPTIMIZED_DELAY */
  return LEG_DEFAULT_DELAY;
#endif /* LEG_USE_OPTIMIZED_DELAY */
}


/*
 * @brief This function moves a leg fully up or down
 * @param legId   leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                will be corrected by legNormalize. Leg ids start to 0
 * @param legUp   must leg goes up or down
 */
void legSetUpDown(BYTE legId,
		  BOOL legUp_)
{
  BYTE position;

  /* compute a valid leg ID */
  legId=legNormalize(legId);
  
  /* set leg position */
  if (legUp_)
    position=legUp[legId];
  else
    position=legDown[legId];
  servoSetPos(legUpDownServo[legId],
	      position);
  
  /* enable if necessary */
  if (!servoIsEnable(legUpDownServo[legId]))
    servoEnable(legUpDownServo[legId], TRUE);
}

/*
 * @brief This function moves a leg fully right or left
 * @param legId     leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                  will be corrected by legNormalize. Leg ids start to 0
 * @param legRight  must leg goes left or right
 */
void legSetRightLeft(BYTE legId,
		     BOOL legRight_)
{
  BYTE position;

  /* compute a valid leg ID */
  legId=legNormalize(legId);
  
  /* set leg position */
  if (legRight_)
    position=legRight[legId];
  else
    position=legLeft[legId];
  servoSetPos(legRightLeftServo[legId],
	      position);
  
  /* enable if necessary */
  if (!servoIsEnable(legRightLeftServo[legId]))
    servoEnable(legRightLeftServo[legId], TRUE);
}

/*
 * @brief This function moves a leg fully inside or outside the robot
 * @param legId   leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                will be corrected by legNormalize. Leg ids start to 0
 * @param legIn   must leg goes left or right
 */
void legSetInOut(BYTE legId,
		 BOOL legIn_)
{
  BYTE position;

  /* compute a valid leg ID */
  legId=legNormalize(legId);
  
  /* set leg position */
  if (legIn_)
    position=legIn[legId];
  else
    position=legOut[legId];
  servoSetPos(legInOutServo[legId],
	      position);
  
  /* enable if necessary */
  if (!servoIsEnable(legInOutServo[legId]))
    servoEnable(legInOutServo[legId], TRUE);
}

/*
 * @brief This function orders the legs to move so that they will touch
 *        the ground
 * @param legMask   legs Id that should touch the ground. Leg 0 is bit 0,
 *                  leg 5 is bit 5
 */
BOOL legsSetTouchTheGround(BYTE legMask)
{
  BYTE legOnTheGround=0x00;
  BYTE i;
  BYTE j;
#ifdef LEG_USE_OPTIMIZED_DELAY
  SBYTE deltaVar=0;
  BYTE  deltaMax=0;
#endif /* LEG_USE_OPTIMIZED_DELAY */

  
#ifdef LEG_DONT_USE_FOOT_BUMPERS  
  /*****************************************************************/
  /*************** LEG_DONT_USE_FOOT_BUMPERS ***********************/
  /*****************************************************************/

  /* move legs that should be on the ground to the middle position */
  for(i=0;i<LEG_NUMBER;++i) {
    j=(1<<i);
    if (j&legMask) {
#ifdef LEG_USE_OPTIMIZED_DELAY
      deltaVar=abs(legMiddle[i]-servoPos[legUpDownServo[i]]);
      if (deltaVar>deltaMax) deltaMax=deltaVar;
#endif /* LEG_USE_OPTIMIZED_DELAY */
      /* move leg to middle position */
      servoSetPos(legUpDownServo[i],
		  legMiddle[i]);
    }
  }
#ifdef LEG_USE_OPTIMIZED_DELAY
  servoWait(deltaMax); 
#else /* LEG_USE_OPTIMIZED_DELAY */
  servoWait(LEG_DEFAULT_DELAY);
#endif /* LEG_USE_OPTIMIZED_DELAY */

  
  /* move legs that should not be on the ground to the upper position */
  for(i=0;i<LEG_NUMBER;++i) {
    j=(1<<i);
    if (!(j&legMask)) {
#ifdef LEG_USE_OPTIMIZED_DELAY
      deltaVar=abs(legMiddle[i]-servoPos[legUpDownServo[i]]);
      if (deltaVar>deltaMax) deltaMax=deltaVar;
#endif /* LEG_USE_OPTIMIZED_DELAY */
      /* move leg to middle position */
      servoSetPos(legUpDownServo[i],
		  legMiddle[i]);
    }
  }
#ifdef LEG_USE_OPTIMIZED_DELAY
  servoWait(deltaMax); 
#else /* LEG_USE_OPTIMIZED_DELAY */
  servoWait(LEG_DEFAULT_DELAY);
#endif /* LEG_USE_OPTIMIZED_DELAY */

  
#else /* LEG_DONT_USE_FOOT_BUMPERS */
  /*****************************************************************/
  /******************** LEG_USE_FOOT_BUMPERS ***********************/
  /*****************************************************************/
  
  while((legOnTheGround=legsAreOnTheGround(legMask)) != legMask) {
    for(i=0;i<LEG_NUMBER;++i) {
      j=(1<<i);
      if (j&legMask) {
	if (servoPos[legUpDownServo[i]]>LEG_DOWN_MAX_FOR_GROUND
	    || !(j&legOnTheGround)) {
	  /* leg that is to high or that is not on the ground but it should be */
	  servoSetPos(legUpDownServo[i],
		      servoPos[legUpDownServo[i]]+LEG_DOWN_DELTA);
	}
#if 0
	else if (legUpDownServo[i] < LEG_DOWN_MIN_FOR_GROUND) {
	  /* leg that is on the ground but maybe to low */
	  servoSetPos(legUpDownServo[i],
		      [legUpDownServo[i]]-LEG_UP_DELTA);
	}
#endif
      }
      
    }
  }
#endif /* LEG_DONT_USE_FOOT_BUMPERS */
  return TRUE;
}


/*
 * @brief This function returns TRUE if a specific leg is touching the ground,
 *        else it returns FALSE.
 *
 * This function uses legsAreOnTheGround, so if you have several call to
 * legIsOnTheGround, for performance you should use legsAreOnTheGround
 * @see legsAreOnTheGround
 */
BOOL legIsOnTheGround(BYTE legId)
{
  BYTE mask = 1<<legId;
  return (legsAreOnTheGround(mask) & mask);
}


/*
 * @brief This function returns a BYTE indicating if specific legs are
 *        touching the ground
 * @param legMask   legs Id that should touch the ground. Leg 0 is bit 0,
 *                  leg 5 is bit 5
 * @return          legs Id that touch the ground. Leg 0 is bit 0,
 *                  leg 5 is bit 5. If bit[i]=0 that means that leg i doesn't
 *                  touch the ground else leg i touches the ground.
 */
BYTE legsAreOnTheGround(BYTE legMask)
{
#ifdef LEG_DONT_USE_FOOT_BUMPERS
  /* by default return that legs are on the ground */
  return legMask;
#else /* LEG_DONT_USE_FOOT_BUMPERS */
  return (ioGetFootBumper()&legMask);
#endif /* LEG_DONT_USE_FOOT_BUMPERS */
}

