/**
 * @file   spiderleg.h
 * @brief  Functions that moves only one leg
 *
 * 
 *
 * @author Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 *
 * @date   2002/09/15
 */

#ifndef SPIDER_LEG_H__
#define SPIDER_LEG_H__

#include "baseTypes.h"

#define LEG_DONT_USE_FOOT_BUMPERS
#define LEG_USE_OPTIMIZED_DELAY

/**
 * @brief default servoWait delay when LEG_USE_OPTIMIZED_DELAY is not set
 */
#define LEG_DEFAULT_DELAY         20

/**
 * @brief Number of legs
 */
#define LEG_NUMBER    6
/**
 * @brief  LEG_NUMBER/2, used by legGetOpposite
 */
#define LEG_NUMBER_2  3 

/**
 * @brief don't modify current servo order when using legSetPosition
 * @see legSetPosition
 */
#define LEG_KEEP      0
/**
 * @brief disable current servo when using legSetPosition
 *
 * The servo controler board stop sending orders to the servo
 * @see legSetPosition
 */
#define LEG_DISABLE   1
/**
 * @brief enable current servo when using legSetPosition
 *
 * The servo controler board start sending orders to the servo
 * corresponding to the latest servo position order
 * @see legSetPosition
 */
#define LEG_ENABLE    2

/**
 * @brief This function sets the servos position for a leg
 * @param legId     Id. It can be invalid(>=LEG_NUMBER) because it
 *                 will be corrected by legNormalize. Leg ids start to 0
 * @param servoUp  servo position. \n
 *                 LEG_ENABLE  = enable servo\n
 *                 LEG_DISABLE = disable servo\n
 *                 LEG_KEEP = don't send any order to the servo\n
 *                 other(>=3) = enable servo and set corresponding position
 * @return Returns the max difference between current position and target 
 *         position for the 3 servo motors (usefull to know how long to wait 
 *         before servos reach their position
 */
BYTE legSetPosition(BYTE legId,
		    BYTE servoUp,
		    BYTE servoRight,
		    BYTE servoIn);

/**
 * @brief This function returns a valid legId corresponding to
 * legId+LEG_NUMBER/2
 */
BYTE legGetOpposite(BYTE legId);

/**
 * @brief This function returns a valid legId between 0 and LEG_NUMBER
 */
BYTE legNormalize(BYTE legId);

/**
 * @brief This function moves a leg fully up or down
 * @param legId   leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                will be corrected by legNormalize. Leg ids start to 0
 * @param legUp   must leg goes up or down
 */
void legSetUpDown(BYTE legId,
		  BOOL legUp);

/**
 * @brief This function moves a leg fully right or left
 * @param legId     leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                  will be corrected by legNormalize. Leg ids start to 0
 * @param legRight  must leg goes left or right
 */
void legSetRightLeft(BYTE legId,
		     BOOL legRight);

/**
 * @brief This function moves a leg fully inside or outside the robot
 * @param legId   leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                will be corrected by legNormalize. Leg ids start to 0
 * @param legIn   must leg goes left or right
 */
void legSetInOut(BYTE legId,
		 BOOL legIn);

/**
 * @brief This function orders the legs to move so that they will touch
 *        the ground
 * @param legMask   legs Id that should touch the ground. Leg 0 is bit 0,
 *                  leg 5 is bit 5
 * @return TRUE if the legs touch the ground, FALSE if it can't make it
 * @todo   Implemement the foot bumper case !
 */
BOOL legsSetTouchTheGround(BYTE legMask);

/**
 * @brief This function returns TRUE if a specific leg is touching the ground,
 *        else it returns FALSE.
 *
 * This function uses legsAreOnTheGround, so if you have several call to
 * legIsOnTheGround, for performance you should use legsAreOnTheGround
 * @see legsAreOnTheGround
 */
BOOL legIsOnTheGround(BYTE legId);

/**
 * @brief This function returns a BYTE indicating if specific legs are
 *        touching the ground
 * @param legMask   legs Id that should touch the ground. Leg 0 is bit 0,
 *                  leg 5 is bit 5
 * @return          legs Id that touch the ground. Leg 0 is bit 0,
 *                  leg 5 is bit 5. If bit[i]=0 that means that leg i doesn't
 *                  touch the ground else leg i touches the ground.
 * @todo implement it
 */
BYTE legsAreOnTheGround(BYTE legMask);

#endif /* SPIDER_LEG_H__ */
