/**
 * @file   spidermove.h
 * @brief  Functions that moves the entire robot
 *
 * @todo Implements the position computation
 *
 * @author Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 *
 * @date   2002/09/15
 */

#ifndef SPIDER_MOVE_H__
#define SPIDER_MOVE_H__

#include "baseTypes.h"


#define MOVE_COMPUTE_POSITION

/**
 * @brief This function moves the robot forward in the mainLegId direction
 * @param mainLegId  leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                   will be corrected by legNormalize. Leg ids start to 0
 * @param dist       approximative distance in cm
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveBackward
 */
BOOL moveForward(BYTE mainLegId,
		 BYTE dist);

/**
 * @brief This function moves the robot backward in the mainLegId direction
 *
 * Same as moveForward, but with the opposite leg !
 * @param mainLegId  leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                   will be corrected by legNormalize. Leg ids start to 0
 * @param dist       approximative distance in cm
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveForward
 */
BOOL moveBackward(BYTE mainLegId,
		  BYTE dist);

/**
 * @brief This function rotates the robot on its left
 * @param angle      approximative angle of rotation in deg
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveRotateRight
 */
BOOL moveRotateLeft(BYTE angle);

/**
 * @brief This function rotates the robot on its right
 * @param angle      approximative angle of rotation in deg
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveRotateLeft
 */
BOOL moveRotateRight(BYTE angle);

/**
 * @brief This function sets the robot in its normal position, all legs
 *        on the ground, near the middle(up/down), center(right/left), normal(in/out),
 *        position
 *
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveStandingByPosition();

/**
 * @brief This function sets the robot in its lowest position.
 *
 * Position of the dying robot !
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveLowerPosition();

/**
 * @brief This function sets the robot in its highest position, all legs
 *        on the ground.
 *
 * Position of the dying robot !
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveUpperPosition();

/**
 * @brief This function moves the legs of the robot like a wave <b>COOL</b> !
 *
 * The robot must be in moveStandingByPosition first !
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveWave(BYTE lapNumber);

/**
 * @brief This function moves the robot so that it can escape a hole !
 *
 * Useful when one leg does not touch the ground !
 * @todo Implement it
 */
void moveEscapeHole();

/**
 * @brief This function returns the robot X coordinates
 * @see moveGetY
 * @see moveGetTheta
 * @see moveSetPosition
 */
SFLOAT moveGetX();
/**
 * @brief This function returns the robot Y coordinates
 * @see moveGetX
 * @see moveGetTheta
 * @see moveSetPosition
 */
SFLOAT moveGetY();
/**
 * @brief This function returns the robot direction in radian
 * @see moveGetX
 * @see moveGetY
 * @see moveSetPosition
 */
SFLOAT moveGetTheta();
/**
 * @brief This function sets the current robot position
 * @see moveGetX
 * @see moveGetY
 * @see moveGetTheta
 */
void  moveSetPosition(SFLOAT x, SFLOAT y, SFLOAT theta);

#endif /* SPIDER_MOVE_H__ */
