/*
 * /file   spidermove.c
 * @brief  Functions that moves the entire robot
 *
 * @description
 *
 * @author Laurent Saint-Marcel (lstmarcel@yahoo.fr)
 *
 * @date   2002/09/15
 */
#ifdef WIN32
#include <stdafx.h>
#endif

#include "servo.h"
#include "spiderleg.h"
#include "spidermove.h"

/* ==================================
 *         SERVO POSITION
 *   must be defined in spiderleg.c
 * ================================== */
/*
 * UP/DOWN
 */
extern const BYTE legUp[LEG_NUMBER];
extern const BYTE legMiddle[LEG_NUMBER];
extern const BYTE legDown[LEG_NUMBER];

/*
 * RIGHT/LEFT
 */
extern const BYTE legLeft[LEG_NUMBER];
extern const BYTE legCenter[LEG_NUMBER];
extern const BYTE legRight[LEG_NUMBER];
/*
 * IN/OUT
 */
extern const BYTE legIn[LEG_NUMBER];
extern const BYTE legNormal[LEG_NUMBER];
extern const BYTE legOut[LEG_NUMBER];

/* ==================================
 *               GLOBALS
 * ================================== */
/* robot position */
static SFLOAT moveX=0;
static SFLOAT moveY=0;
static SFLOAT moveTheta=0;

static BOOL  moveIsInStandByPosition=FALSE;

/* legs A = legs 0, 2, 4
   legs B = legs 1, 3, 5

           0
	   |
       1__/ \__5
	_|   |_
       2  \ /  4
	   |
	   3
*/
#define MASK_LEGS_A   0x15
#define MASK_LEGS_B   0x2A

/* Time needed to move legs from on position to another
   See servoWait
 */
#define MOVE_LEG_MIDDLE_UP   25
#define MOVE_LEG_UP_DOWN     50
#define MOVE_LEG_DOWN_MIDDLE 25
#define MOVE_LEG_RIGHT_LEFT  40

/* to compute robot position */
#define MOVE_DIST_FORWARD_STEP 5. /* cm */
#define MOVE_ANGLE_ROTATE_STEP .1 /* rad */

/* ==================================
 *              FUNCTIONS
 * ================================== */

/*
 * @brief This function moves the robot forward in the mainLegId direction
 * @param mainLegId  leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                   will be corrected by legNormalize. Leg ids start to 0
 * @param dist       approximative distance in cm
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveBackward
 */
BOOL moveForward(BYTE mainLegId,
		 BYTE dist)
{
  BYTE maxVar=0x00;
  BYTE deltaVar;
  BYTE legId=0;
  BYTE legsAMask=0; /*legs 0 2 4 */
  BYTE legsBMask=0; /*legs 1 3 5 */
  BYTE i;
  
#ifdef MOVE_COMPUTE_POSITION
  SFLOAT dir;
  SFLOAT deltaX, deltaY;
  dir=legNormalize(mainLegId)*M_PI_3+moveTheta;
  deltaX=MOVE_DIST_FORWARD_STEP*cos(dir);
  deltaY=MOVE_DIST_FORWARD_STEP*sin(dir);
#endif
  
  /* make sure to be in standing by position */
  if (!moveStandingByPosition())
    return FALSE;

  moveIsInStandByPosition=FALSE;
  
  /* compute leg mask to check legs on the ground */
  legsAMask=(1<<(legNormalize(mainLegId)))
    +(1<<(legNormalize(mainLegId+2)))
    +(1<<(legNormalize(mainLegId+4)));
  legsBMask=(1<<(legNormalize(mainLegId+1)))
    +(1<<(legNormalize(mainLegId+3)))
    +(1<<(legNormalize(mainLegId+5)));

  /* legs B down */
  legSetPosition(1, legDown[1], LEG_KEEP, LEG_KEEP);
  legSetPosition(3, legDown[3], LEG_KEEP, LEG_KEEP);
  legSetPosition(5, legDown[5], LEG_KEEP, LEG_KEEP);
  servoWait(MOVE_LEG_UP_DOWN);
  
  /* Do dist x steps */
  for(i=0;i<dist;++i) {
    
    /* move legs A up and forward*/
    legId=mainLegId;
    legSetPosition(legId,legUp[legId],legCenter[legId],legOut[legId]);
    legId=mainLegId+2;
    legSetPosition(legId,legUp[legId],legRight[legId],legIn[legId]);
    legId=mainLegId+4;
    legSetPosition(legId,legUp[legId],legLeft[legId],legIn[legId]);
    servoWait(MOVE_LEG_RIGHT_LEFT);

    /* save power by disabling servo for legs on the ground (legs A) */
    legSetPosition(mainLegId,  LEG_KEEP,LEG_DISABLE,LEG_DISABLE); 
    legSetPosition(mainLegId+2,LEG_KEEP,LEG_DISABLE,LEG_DISABLE); 
    legSetPosition(mainLegId+4,LEG_KEEP,LEG_DISABLE,LEG_DISABLE);
    
    /* move legs B backward.*/
    legId=mainLegId+1;
    legSetPosition(legId,LEG_KEEP,legLeft[legId],LEG_KEEP);
    legId=mainLegId+3;
    legSetPosition(legId,LEG_KEEP,LEG_KEEP,legOut[legId]);
    legId=mainLegId+5;
    legSetPosition(legId,LEG_KEEP,legRight[legId],LEG_KEEP);
    servoWait(MOVE_LEG_RIGHT_LEFT);

    /* update robot position */
#ifdef MOVE_COMPUTE_POSITION
    moveX += deltaX;
    moveY += deltaY;
#endif
    
    /* save power by disabling servo for legs on the ground (legs B) */
    legSetPosition(mainLegId+1, LEG_KEEP, LEG_DISABLE, LEG_KEEP);
    legSetPosition(mainLegId+3, LEG_KEEP, LEG_KEEP, LEG_DISABLE);
    legSetPosition(mainLegId+5, LEG_KEEP, LEG_DISABLE, LEG_KEEP);
    
    /* move legs A down */
    legId=mainLegId;
    legSetPosition(legId,legDown[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+2;
    legSetPosition(legId,legDown[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+4;
    legSetPosition(legId,legDown[legId],LEG_KEEP,LEG_KEEP);
    servoWait(MOVE_LEG_UP_DOWN);

    /* move legs B to middle position */
    legId=mainLegId+1;
    legSetPosition(legId,legMiddle[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+3;
    legSetPosition(legId,legMiddle[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+5;
    legSetPosition(legId,legMiddle[legId],LEG_KEEP,LEG_KEEP);
    servoWait(MOVE_LEG_DOWN_MIDDLE);

    /* check if legs A are on the ground */
    if (legsAreOnTheGround(legsAMask)!=legsAMask)
      return FALSE; /* at least one leg is not on the ground => problem */

    /* move legs B to upper position and forward */
    legId=mainLegId+1;
    legSetPosition(legId,legUp[legId],legRight[legId],legOut[legId]);
    legId=mainLegId+3;
    legSetPosition(legId,legUp[legId],legCenter[legId],legIn[legId]);
    legId=mainLegId+5;
    legSetPosition(legId,legUp[legId],legLeft[legId],legOut[legId]);
    servoWait(MOVE_LEG_RIGHT_LEFT);

    /* save power by disabling servo for legs on the ground (legs B) */
    legSetPosition(mainLegId+1, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
    legSetPosition(mainLegId+3, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
    legSetPosition(mainLegId+5, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);

    /* move legs A backward.*/
    legId=mainLegId;
    deltaVar=legSetPosition(legId,LEG_KEEP,LEG_KEEP,legIn[legId]);
    legId=mainLegId+2;
    deltaVar=legSetPosition(legId,LEG_KEEP,legLeft[legId],LEG_KEEP);
    legId=mainLegId+4;
    deltaVar=legSetPosition(legId,LEG_KEEP,legRight[legId],LEG_KEEP);
    servoWait(MOVE_LEG_RIGHT_LEFT);

    /* update robot position */
#ifdef MOVE_COMPUTE_POSITION
    moveX += deltaX;
    moveY += deltaY;
#endif
    
    /* save power by disabling servo for legs on the ground (legs A) */
    legSetPosition(mainLegId,  LEG_KEEP,LEG_KEEP,LEG_DISABLE); 
    legSetPosition(mainLegId+2,LEG_KEEP,LEG_DISABLE,LEG_KEEP); 
    legSetPosition(mainLegId+4,LEG_KEEP,LEG_DISABLE,LEG_KEEP);
    
     /* move legs B down */
    legId=mainLegId+1;
    legSetPosition(legId,legDown[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+3;
    legSetPosition(legId,legDown[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+5;
    legSetPosition(legId,legDown[legId],LEG_KEEP,LEG_KEEP);
    servoWait(MOVE_LEG_UP_DOWN);

     /* move legs A to middle position */
    legId=mainLegId;
    legSetPosition(legId,legMiddle[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+2;
    legSetPosition(legId,legMiddle[legId],LEG_KEEP,LEG_KEEP);
    legId=mainLegId+4;
    legSetPosition(legId,legMiddle[legId],LEG_KEEP,LEG_KEEP);
    servoWait(MOVE_LEG_DOWN_MIDDLE);

    /* check if legs B are on the ground */
    if (legsAreOnTheGround(legsBMask)!=legsBMask)
      return FALSE; /* at least one leg is not on the ground => problem */
    
  }
  
  /* go back to standingBy position */
  return moveStandingByPosition();
}


/*
 * @brief This function moves the robot backward in the mainLegId direction
 *
 * Same as moveForward, but with the opposite leg !
 * @param mainLegId  leg Id. It can be invalid(>=LEG_NUMBER) because it
 *                   will be corrected by legNormalize. Leg ids start to 0
 * @param dist       approximative distance in cm
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveForward
 */
BOOL moveBackward(BYTE mainLegId,
		  BYTE dist)
{
  return moveForward(legGetOpposite(mainLegId), dist);
}


/*
 * @brief This function rotates the robot on its left or right
 *
 * == PRIVATE FUNCTION ==
 *
 * @param angle      approximative angle of rotation in deg
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveRotateRight, moveRotateLeft
 */
BOOL moveRotate(BYTE angle,
		BOOL rotateLeft)
{
  BYTE maxVar=0x00;
  BYTE deltaVar;
  BYTE legId=0;
  BYTE i;
  
  /* make sure to be in standing by position */
  if (!moveStandingByPosition())
    return FALSE;

  moveIsInStandByPosition=FALSE;
  
  /* legs B down */
  legSetPosition(1, legDown[1], LEG_KEEP, LEG_KEEP);
  legSetPosition(3, legDown[3], LEG_KEEP, LEG_KEEP);
  legSetPosition(5, legDown[5], LEG_KEEP, LEG_KEEP);
  servoWait(MOVE_LEG_UP_DOWN);
  
  /* Do angle x steps */
  for(i=0;i<angle;++i) {
    /* save power by disabling servo for legs on the ground (legs B) */
    legSetPosition(1, LEG_KEEP, LEG_DISABLE, LEG_KEEP);
    legSetPosition(3, LEG_KEEP, LEG_DISABLE, LEG_KEEP);
    legSetPosition(5, LEG_KEEP, LEG_DISABLE, LEG_KEEP);
    
    if (rotateLeft) {
      /* move legs A up and left*/
      legSetPosition(0,legUp[0],legLeft[0],LEG_KEEP);
      legSetPosition(2,legUp[2],legLeft[2],LEG_KEEP);
      legSetPosition(4,legUp[4],legLeft[4],LEG_KEEP);
      /*  servoWait(MOVE_LEG_RIGHT_LEFT);*/
      
      /* move legs B right.*/
      legSetPosition(1,LEG_KEEP,legRight[1],LEG_KEEP);
      legSetPosition(3,LEG_KEEP,legRight[3],LEG_KEEP);
      legSetPosition(5,LEG_KEEP,legRight[5],LEG_KEEP);
      servoWait(MOVE_LEG_RIGHT_LEFT);
      
#ifdef MOVE_COMPUTE_POSITION
      moveTheta+=MOVE_ANGLE_ROTATE_STEP;
#endif
    } else { /* rotate right */
      /* move legs A up and right*/
      legSetPosition(0,legUp[0],legRight[0],LEG_KEEP);
      legSetPosition(2,legUp[2],legRight[2],LEG_KEEP);
      legSetPosition(4,legUp[4],legRight[4],LEG_KEEP);
      /*  servoWait(MOVE_LEG_RIGHT_LEFT);*/
      
      /* move legs B left.*/
      legSetPosition(1,LEG_KEEP,legLeft[1],LEG_KEEP);
      legSetPosition(3,LEG_KEEP,legLeft[3],LEG_KEEP);
      legSetPosition(5,LEG_KEEP,legLeft[5],LEG_KEEP);
      servoWait(MOVE_LEG_RIGHT_LEFT);
      
#ifdef MOVE_COMPUTE_POSITION
      moveTheta-=MOVE_ANGLE_ROTATE_STEP;
#endif
    }
    
    
    /* move legs A down */
    legSetPosition(0,legDown[0],LEG_DISABLE,LEG_KEEP);
    legSetPosition(2,legDown[2],LEG_DISABLE,LEG_KEEP);
    legSetPosition(4,legDown[4],LEG_DISABLE,LEG_KEEP);
    servoWait(MOVE_LEG_UP_DOWN);

    /* move legs B middle.*/
    legSetPosition(1,legMiddle[1],LEG_DISABLE,LEG_KEEP);
    legSetPosition(3,legMiddle[3],LEG_DISABLE,LEG_KEEP);
    legSetPosition(5,legMiddle[5],LEG_DISABLE,LEG_KEEP);
    servoWait(MOVE_LEG_DOWN_MIDDLE);

    /* check if legs A are on the ground */
    if (legsAreOnTheGround(MASK_LEGS_A)!=MASK_LEGS_A)
      return FALSE; /* at least one leg is not on the ground => problem */

    if (rotateLeft) {
      /* move legs A up and left*/
      legSetPosition(1,legUp[1],legLeft[1],LEG_KEEP);
      legSetPosition(3,legUp[3],legLeft[3],LEG_KEEP);
      legSetPosition(5,legUp[5],legLeft[5],LEG_KEEP);
      /*  servoWait(MOVE_LEG_RIGHT_LEFT);*/
      
      /* move legs B right.*/
      legSetPosition(0,LEG_KEEP,legRight[0],LEG_KEEP);
      legSetPosition(2,LEG_KEEP,legRight[2],LEG_KEEP);
      legSetPosition(4,LEG_KEEP,legRight[4],LEG_KEEP);
      servoWait(MOVE_LEG_RIGHT_LEFT);

#ifdef MOVE_COMPUTE_POSITION
      moveTheta+=MOVE_ANGLE_ROTATE_STEP;
#endif
    } else { /* rotate right */
      /* move legs A up and right*/
      legSetPosition(1,legUp[1],legRight[1],LEG_KEEP);
      legSetPosition(3,legUp[3],legRight[3],LEG_KEEP);
      legSetPosition(5,legUp[5],legRight[5],LEG_KEEP);
      /*  servoWait(MOVE_LEG_RIGHT_LEFT);*/
      
      /* move legs B left.*/
      legSetPosition(0,LEG_KEEP,legLeft[0],LEG_KEEP);
      legSetPosition(2,LEG_KEEP,legLeft[2],LEG_KEEP);
      legSetPosition(4,LEG_KEEP,legLeft[4],LEG_KEEP);
      servoWait(MOVE_LEG_RIGHT_LEFT);

#ifdef MOVE_COMPUTE_POSITION
      moveTheta-=MOVE_ANGLE_ROTATE_STEP;
#endif
    }
    
    /* move legs B down */
    legSetPosition(1,legDown[1],LEG_DISABLE,LEG_KEEP);
    legSetPosition(3,legDown[3],LEG_DISABLE,LEG_KEEP);
    legSetPosition(5,legDown[5],LEG_DISABLE,LEG_KEEP);
    servoWait(MOVE_LEG_UP_DOWN);

    /* move legs A middle.*/
    legSetPosition(0,legMiddle[0],LEG_DISABLE,LEG_KEEP);
    legSetPosition(2,legMiddle[2],LEG_DISABLE,LEG_KEEP);
    legSetPosition(4,legMiddle[4],LEG_DISABLE,LEG_KEEP);
    servoWait(MOVE_LEG_DOWN_MIDDLE);

     /* check if legs B are on the ground */
    if (legsAreOnTheGround(MASK_LEGS_B)!=MASK_LEGS_B)
      return FALSE; /* at least one leg is not on the ground => problem */
  }
  
  /* go back to standingBy position */
  return moveStandingByPosition();
}

/*
 * @brief This function rotates the robot on its left
 * @param angle      approximative angle of rotation in deg
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveRotateRight
 */
BOOL moveRotateLeft(BYTE angle)
{
  return moveRotate(angle, TRUE);
}

/*
 * @brief This function rotates the robot on its right
 * @param angle      approximative angle of rotation in deg
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 * @see moveRotateLeft
 */
BOOL moveRotateRight(BYTE angle)
{
  return moveRotate(angle, FALSE);
}


/*
 * @brief This function sets the robot in its lowest position.
 *
 * Position of the dying robot !
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveLowerPosition()
{
  BYTE i;
  BYTE maxVar=0x00;
  BYTE deltaVar;
  moveIsInStandByPosition=FALSE;
  
  for(i=0;i<LEG_NUMBER;++i){
    deltaVar=legSetPosition(i,legDown[i], LEG_DISABLE, LEG_DISABLE);
    if (deltaVar>maxVar) maxVar=deltaVar;
  }
  servoWait(maxVar);
  return TRUE;
}

/*
 * @brief This function moves legs A to the standby position
 *
 * == PRIVATE FUNCTION ==
 * Move legs A up/center/normal then make them touch the ground.
 *
 * @return   if all legs A touch the ground, return TRUE, else return FALSE
 *
 * @see      legsSetTouchTheGround
 */
BOOL moveSetStandingByLegsA()
{
  BYTE i;
  BYTE maxVar=0x00;
  BYTE deltaVar;

  /* move legs A up and center/normal position */
  maxVar=0;
  for(i=0;i<LEG_NUMBER;i+=2) {
    deltaVar=legSetPosition(i,legUp[i], legCenter[i], legNormal[i]);
    if (deltaVar>maxVar) maxVar=deltaVar;
  }
  servoWait(maxVar);

  /* disable some servo to save power */
  legSetPosition(0,LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(2,LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(4,LEG_KEEP, LEG_DISABLE, LEG_DISABLE);

  /* move legs A down */
  if (!legsSetTouchTheGround(MASK_LEGS_A))
    return FALSE;
  else
    return TRUE;
}

/*
 * @brief This function moves legs B to the standby position
 *
 * == PRIVATE FUNCTION ==
 * Move legs B up/center/normal then make them touch the ground.
 *
 * @return   if all legs B touch the ground, return TRUE, else return FALSE
 *
 * @see      legsSetTouchTheGround
 * @see      moveStandingByPosition
 */
BOOL moveSetStandingByLegsB()
{
  BYTE i;
  BYTE maxVar=0x00;
  BYTE deltaVar;

  /* move legs A up and center/normal position */
  maxVar=0;
  for(i=1;i<LEG_NUMBER;i+=2) {
    deltaVar=legSetPosition(i,legUp[i], legCenter[i], legNormal[i]);
    if (deltaVar>maxVar) maxVar=deltaVar;
  }
  servoWait(maxVar);

  /* disable some servo to save power */
  legSetPosition(1,LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(3,LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(5,LEG_KEEP, LEG_DISABLE, LEG_DISABLE);

  /* move legs A down */
  if (!legsSetTouchTheGround(MASK_LEGS_B))
    return FALSE;
  else
    return TRUE;
}


/*
 * @brief This function sets the robot in its normal position, all legs
 *        on the ground, near the middle(up/down), center(right/left),
 *        normal(in/out) position
 *
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveStandingByPosition()
{
  BYTE i;
  BYTE maxVar=0x00;
  BYTE deltaVar;
  if (moveIsInStandByPosition)
    return TRUE;

  /* set legs in middle position */
  for(i=0;i<LEG_NUMBER;++i) {
    deltaVar=legSetPosition(i,legMiddle[i], LEG_DISABLE, LEG_DISABLE);
    if (deltaVar>maxVar) maxVar=deltaVar;
  }
  servoWait(maxVar);

  /* move legs to center/normal position */
  if (!legsSetTouchTheGround(MASK_LEGS_B)) {
    /* legs B can't touch the ground, check legs A */
    if (legsSetTouchTheGround(MASK_LEGS_A)) {
      /* legs A touch the ground => move legs B then legs A */
      if (!moveSetStandingByLegsB()
	  || !moveSetStandingByLegsA())
	return FALSE;
    } else {
      /* some legs from A and B can't touch the ground => problem */
      return FALSE;
    }
  } else {
    /* legs B touch the ground => move legs A then legs B */
    if (!moveSetStandingByLegsA()
	|| !moveSetStandingByLegsB())
      return FALSE;
  }
  
  /* disable right/left and in/out */
  legSetPosition(0, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(1, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(2, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(3, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(4, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  legSetPosition(5, LEG_KEEP, LEG_DISABLE, LEG_DISABLE);
  
  moveIsInStandByPosition=TRUE;
  return TRUE;
}

/*
 * @brief This function sets the robot in its lowest position.
 *
 * Position of the dying robot !
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveLowerPosition()
{
  BYTE i;
  BYTE maxVar=0x00;
  BYTE deltaVar;
  moveIsInStandByPosition=FALSE;
  
  for(i=0;i<LEG_NUMBER;++i){
    deltaVar=legSetPosition(i,legDown[i], LEG_DISABLE, LEG_DISABLE);
    if (deltaVar>maxVar) maxVar=deltaVar;
  }
  servoWait(maxVar);
  return TRUE;
}

/*
 * @brief This function sets the robot in its highest position, all legs
 *        on the ground.
 *
 * Position of the dying robot !
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveUpperPosition()
{
  BYTE i;
  BYTE maxVar=0x00;
  BYTE deltaVar;
  moveIsInStandByPosition=FALSE;
  
  for(i=0;i<LEG_NUMBER;++i){
    deltaVar=legSetPosition(i,legUp[i], LEG_DISABLE, LEG_DISABLE);
    if (deltaVar>maxVar) maxVar=deltaVar;
  }
  servoWait(maxVar);
  return TRUE;
}


/*
 * @brief This function moves the legs of the robot like a wave <b>COOL</b> !
 * @return           it returns TRUE if the movement terminate normaly, else
 *                   (collision, leg not on the ground...) it return FALSE
 */
BOOL moveWave(BYTE lapNumber)
{
  BYTE i,j;
  /* make sure to be in standing by position */
  if (!moveStandingByPosition())
    return FALSE;
  
  legSetPosition(i,legUp[0], LEG_DISABLE, LEG_DISABLE);
  sleep01(10);
  for(j=0;j<lapNumber;++j) {
    for(i=0;i<LEG_NUMBER;++i) { 
      legSetPosition(i,legMiddle[i], LEG_DISABLE, LEG_DISABLE);
      legSetPosition(i,legUp[legNormalize(i+1)], LEG_DISABLE, LEG_DISABLE);
      sleep01(10);
    }
    legSetPosition(0,legMiddle[0], LEG_DISABLE, LEG_DISABLE);
    sleep01(10);
  }
  return TRUE;
}

/*
 * @brief This function moves the robot so that it can escape a hole !
 */
void moveEscapeHole()
{
  /* TODO implement it */
}

/*
 * @brief This function returns the robot X coordinates
 * @see moveGetY
 * @see moveGetTheta
 * @see moveSetPosition
 */
SFLOAT moveGetX()
{
  return moveX;
}


/*
 * @brief This function returns the robot Y coordinates
 * @see moveGetX
 * @see moveGetTheta
 * @see moveSetPosition
 */
SFLOAT moveGetY()
{
  return moveY;
}

/*
 * @brief This function returns the robot direction in radian
 * @see moveGetX
 * @see moveGetY
 * @see moveSetPosition
 */
SFLOAT moveGetTheta()
{
  return moveTheta;
}


/*
 * @brief This function sets the current robot position
 * @see moveGetX
 * @see moveGetY
 * @see moveGetTheta
 */
void  moveSetPosition(SFLOAT x, SFLOAT y, SFLOAT theta)
{
  moveTheta = theta;
  moveX = x;
  moveY = y;
}

